#! /usr/bin/env python

import gd, sys, time
from math import exp
from os import system
from optparse import OptionParser

class mandelbrotSet:

    def __init__(self):
        print 'created Mandelbrot set'

    def iter(self, z):
        c = z
        for i in range(maxIter):
            z = z*z + c
            if abs(z) > bailout:
                return i
        return maxIter

class juliaSet:

    def __init__(self, jc):
        self.c = jc
        print 'created Julia set at', self.c

    def iter(self, z):
        for i in range(maxIter):
            z = z*z + self.c
            if abs(z) > bailout:
                return i
        return maxIter

class graph:

    jpgQuality = 95

    def gauss(self, x, mu, sigma):
        return int(255*exp(-(x-mu)*(x-mu)/(sigma*sigma)))

    def setColors(self):
        for i in range(nColors):
            r = self.gauss(i,  75.0, 60.0)
            g = self.gauss(i, 125.0, 60.0)
            b = self.gauss(i, 180.0, 60.0)
            self.canvas.colorAllocate((r, g, b))

    def plot(self):
	timeStart = time.clock()
        dx = self.width/self.pixels
        bl = self.center - complex(self.width/2, self.width/2)
        print 'plotting graph: center:', self.center, 'width:', self.width
        for ix in range(self.pixels):
            if (ix % 10 == 0):
                print '\rstarting row %03i' % ix,
                sys.stdout.flush()
            for iy in range(self.pixels):
                c = bl + complex(ix*dx, iy*dx)
                color = self.fractalSet.iter(c)
                self.canvas.setPixel((ix, self.pixels-1 - iy), color)
	timeStop = time.clock()
	timeDiff = round(timeStop-timeStart, 3)
        self.canvas.writeJpeg(jpgFile, self.jpgQuality)
        print '\rfile', jpgFile, 'has been created in', timeDiff, 'seconds'

    def halfPlot(self):
        # plot a Julia set using its symetry, if center==0
	timeStart = time.clock()
        dx = self.width/self.pixels
        bl = self.center - complex(self.width/2, self.width/2)
        print 'plotting symmetric Julia set: center:', self.center, 'width:', self.width
        for ix in range(self.pixels):
            if (ix % 10 == 0):
                print '\rstarting row %03i' % ix,
                sys.stdout.flush()
            for iy in range(self.pixels/2):
                c = bl + complex(ix*dx, iy*dx)
                color = self.fractalSet.iter(c)
                self.canvas.setPixel((ix, self.pixels-1 - iy), color)
                self.canvas.setPixel((self.pixels-1 - ix, iy), color)
	timeStop = time.clock()
	timeDiff = round(timeStop-timeStart, 3)
        self.canvas.writeJpeg(jpgFile, self.jpgQuality)
        print '\rfile', jpgFile, 'has been created in', timeDiff, 'seconds'

    def __init__(self, tset, tpixels=100, twidth=3.0, tcenter=complex(0,0)):
        self.center = tcenter
        self.width = twidth
        self.pixels = tpixels
        self.fractalSet = tset
        self.canvas = gd.image((self.pixels, self.pixels))
        self.setColors()
        print 'created graph: size', self.pixels, 'pixels' 

# main ...

print "fractals by nicZ"

# options
parser = OptionParser()
parser.add_option("-f", "--file", dest="filename", metavar="FILE", 
                  default='fractal.jpg', help="write image to FILE")
parser.add_option("-s", "--size", type="int", dest="size",
                  help="size of image in pixels", default=100)
parser.add_option("-c", "--center", type="complex", dest="center",
                  help="complex center of image, eg 0.1-0.2j", default=complex(0, 0))
parser.add_option("-w", "--width", type="float", dest="width",
                  help="width of image window", default=3.0)
parser.add_option("-j", "--julia", type="complex", dest="cJulia", metavar="Z", 
                  help="plot the Julia set for complex Z, instead of the Mandelbrot set") 

(options, args) = parser.parse_args()

bailout = 2.0
maxIter = 250
nColors = maxIter + 1
jpgFile = options.filename

if options.cJulia==None:
    # Mandelbrot set
    mySet = mandelbrotSet()
    myGraph = graph(mySet, options.size, options.width, options.center)
    myGraph.plot()
else:
    # Julia set
    mySet = juliaSet(options.cJulia)
    myGraph = graph(mySet, options.size, options.width, options.center)
    if options.center==complex(0,0) : myGraph.halfPlot()
    else: myGraph.plot()

system('display '+jpgFile+' &')
print

# Mandelbrot set images : center, width
# (-1.278361, 0.06548), 0.0002   : etincelle
# (-0.76043,  0.11301), 0.06     : vallee
# (-1.4858,   0.0372),  0.0025   : mini ilot
# (-1.478,    0),       0.038    : ilot secondaire
# (-0.12,     0.84),    0.5      : bourgeon nord
# ( 0.34949,  0.06657), 0.002    : ilot 4 branches
# ( 0.34665, -0.06982), 0.03     : bourgeon
# (-0.65,     0),       2.4      : tout
# (-0.73756, -0.16869), 0.0006   : spirale
# (-1.27966, 0.064341), 0.001386 : hexapode



